void leaf1_vp( 
	float4 position	: POSITION,
	float2 uv		: TEXCOORD0,
	float3 normal	: NORMAL,
	float3 tangent  : TEXCOORD1,
	float4 v_color	: COLOR0,

	out float2 oUv			: TEXCOORD0,
	out float4 oPosition	: POSITION,
	
	out float3 oeyeDirTS		: TEXCOORD2,
	out float3 olightDirTS		: TEXCOORD3,
	out float3 onormalTS		: TEXCOORD4,

	
	uniform float4x4 worldViewProj,
	uniform float4 eyePosition,
	uniform float4 lightPosition

	)

{
	
	oPosition = mul(worldViewProj, position);
	oUv = uv;	

	float3 binormal = cross(normal,tangent);
	float3x3 TBN = float3x3(tangent, binormal, normal);

	
	oeyeDirTS = (eyePosition-position);
	oeyeDirTS = mul(TBN,oeyeDirTS);
	
	olightDirTS = (lightPosition - position);
	olightDirTS = mul(TBN,olightDirTS);
	
	onormalTS = mul(TBN,normal);
	
}


float D(float cosa, float m)
{
return (1.0f/(pow(m, 2)*pow(cosa, 4))) * 
	        exp(-((1.0f/pow(cosa, 2)-1.0f)/pow(m, 2)));
}

//This function computes the shadowing and masking term
float G(float3 N, float3 H, float3 V, float3 L)
{
	float m = min(2.0f*dot(N,H)*dot(N,V)/dot(V,H),2.0f*dot(N,H)*dot(N,L)/dot(V,H));
	return min(1.0f, m);
}

//This function computes the fresnel term
float F(float n, float c)
{
	float g = sqrt(n*n+c*c-1);
	return (0.5f)*(pow((g-c), 2)/pow((g+c), 2)) * 
	       (1+pow((c*(g+c)-1), 2)/pow((c*(g-c)+1), 2));
}



void leaf1_fp(float2 uv				: TEXCOORD0,
		      float facing		    : VFACE,
		      float3 eyeDirTS       : TEXCOORD2,
		      float3 lightDirTS		: TEXCOORD3,
		      float3 normalTS		:TEXCOORD4,

		 
			  out float4 color	: COLOR,
			  
			  uniform float4 ambient,
			 
			  uniform sampler2D leaf_front,
			  uniform sampler2D leaf_transl_front,
			  uniform sampler2D leaf_TSNM_front,
			  uniform sampler2D leaf_TSHLM_front_t,
			   
			  uniform sampler2D leaf_back,
			  uniform sampler2D leaf_transl_back,
			  uniform sampler2D leaf_TSNM_back,
			  uniform sampler2D leaf_TSHLM_back_t
			  
			  )
{
	
	float4 c_out;
	
	lightDirTS = normalize(lightDirTS);
	eyeDirTS = normalize(eyeDirTS);	
	normalTS = normalize(normalTS);
	
	
	if(dot(normalTS,eyeDirTS) >0.0)
	{
	
		float4 normalmapTS_front = (tex2D(leaf_TSNM_front,uv)-0.5)*2;
		float4 normalmapTS_back = (tex2D(leaf_TSNM_back,uv)-0.5)*2;
			
		//diffuse
				
		float shade_front = max(dot(lightDirTS,normalmapTS_front),0);
		
		float4 halflife_TS_front_t = (tex2D(leaf_TSHLM_front_t,uv));	
		
		
#define SQRT6  0.40824829046386301636621401245098
#define SQRT2  0.70710678118654752440084436210485
#define SQRT3  0.57735026918962576450914878050196
#define SQRT23 0.81649658092772603273242802490196		
		

			
		float shade_front_t = dot(lightDirTS,float3(-SQRT6,-SQRT2,-SQRT3))*halflife_TS_front_t.x +
				      dot(lightDirTS,float3(-SQRT6,SQRT2,-SQRT3))*halflife_TS_front_t.y +
				      dot(lightDirTS,float3(SQRT23,0,-SQRT3))*halflife_TS_front_t.z ;
				      	      
				   
				      
#define REF_INDEX 1.6
#define ROUGHNESS 0.4

		float3 wHalfAngleDir = normalize(lightDirTS + eyeDirTS);
		float fresnel = F(REF_INDEX,dot(eyeDirTS,wHalfAngleDir));
		float rough = D(dot(normalmapTS_front,wHalfAngleDir),ROUGHNESS);
		
		float specularCoefficient = (rough/dot(normalmapTS_front,lightDirTS))*max(0,
		     (G(normalmapTS_front,wHalfAngleDir,eyeDirTS,lightDirTS)/dot(normalmapTS_front,eyeDirTS))*fresnel/3.14);
		     
		     
			    	     
		float4 front = tex2D(leaf_front,uv);
		float4 front_t = tex2D(leaf_transl_front,uv);
		
						
		c_out.xyz = (front.xyz*(max(0,shade_front)+ambient)+ front_t.xyz*(max(0,shade_front_t)))+specularCoefficient;	
		c_out.w = front.w;
				
		
	}
	else
	{
			
		float4 normalmapTS_back = (tex2D(leaf_TSNM_back,uv)-0.5)*2;
		
		normalmapTS_back.x = -normalmapTS_back.x;
		normalmapTS_back.y = -normalmapTS_back.y;
		
			
		//diffuse
				
		float shade_back = max(dot(-lightDirTS,normalmapTS_back),0);

		
		float4 halflife_TS_back_t = (tex2D(leaf_TSHLM_back_t,uv));	
	
		float shade_back_t = dot(lightDirTS,float3(-SQRT6,-SQRT2,SQRT3))*halflife_TS_back_t.x +
							  dot(lightDirTS,float3(-SQRT6,SQRT2,SQRT3))*halflife_TS_back_t.y +
							  dot(lightDirTS,float3(SQRT23,0,SQRT3))*halflife_TS_back_t.z ;
							  

		
		float4 back = tex2D(leaf_back,uv);
		float4 back_t = tex2D(leaf_transl_back,uv);
			
		
		c_out.xyz =back.xyz*max(0,shade_back) + back_t.xyz*max(0,shade_back_t);
		c_out.w = back.w;
	}
	
	color = c_out;
	

}